package cn.sh.stc.sict.theme.hphy.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.sh.stc.sict.cloud.common.core.constant.Constant;
import cn.sh.stc.sict.cloud.common.core.dto.*;
import cn.sh.stc.sict.cloud.common.core.util.NumberUtil;
import cn.sh.stc.sict.cloud.common.security.util.SecurityUtils;
import cn.sh.stc.sict.cloud.upms.dto.CurrentUser;
import cn.sh.stc.sict.theme.hphy.dao.HphyPatientBaseMapper;
import cn.sh.stc.sict.theme.hphy.model.HphyPatientBase;
import cn.sh.stc.sict.theme.hphy.service.HphyPatientBaseService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.List;


@Service("hphyPatientBaseService")
@AllArgsConstructor
public class HphyPatientBaseServiceImpl extends ServiceImpl<HphyPatientBaseMapper, HphyPatientBase> implements HphyPatientBaseService {

    private final HphyPatientBaseMapper hphyPatientBaseMapper;

    @Override
    public HphyPatientBase getByOpenId(String openId) {
        LambdaQueryWrapper<HphyPatientBase> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(HphyPatientBase::getOpenId, openId);

        return this.getOne(wrapper);
    }

    @Override
    public HphyPatientBase getByPhone(String phone, Boolean flag) {
        LambdaQueryWrapper<HphyPatientBase> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(HphyPatientBase::getPhone, phone)
                .isNull(flag, HphyPatientBase::getOpenId)
                .orderByDesc(HphyPatientBase::getUpdateTime)
                .last("limit 1");

        return this.getOne(wrapper);
    }

    @Override
    public HphyPatientBase saveSSbInfo(CurrentUser current, WDUserInfo userInfo, List<WDUserCardInfo> cardList) {
        if (ObjectUtil.isNotNull(userInfo)) {
            HphyPatientBase base = new HphyPatientBase();
            base.setUserId(current.getId());
            base.setOpenId(current.getOpenId());
            base.setPhone(userInfo.getMobile());
            base.setName(userInfo.getName());
            base.setCertId(userInfo.getPersoncard());
            base.setAge(String.valueOf(IdcardUtil.getAgeByIdCard(userInfo.getPersoncard())));
            base.setGender(Constant.BYTE_YES.equals(userInfo.getGender()) ? "男" : "女");
            base.setVerifyStatus(userInfo.getVerifyStatus());
            if (CollUtil.isNotEmpty(cardList)) {
                WDUserCardInfo card = cardList.get(0);
                base.setMediCardId(card.getCardNo());
                base.setMediCardType(card.getCardType());
            }

            this.save(base);
            return base;
        }
        return null;
    }

    /**
     * 保存香山中医医院公众号用户
     *
     * @param current  当前用户
     * @param userInfo 公众号用户详情
     * @return XSZYUserInfo {@link XSZYUserInfo}
     */

    @Override
    public HphyPatientBase saveWoaXszyInfo(CurrentUser current, XSZYUserInfo userInfo) {
        if (ObjectUtil.isNotNull(userInfo)) {
            HphyPatientBase base = new HphyPatientBase();
            base.setUserId(current.getId());
            base.setOpenId(current.getOpenId());
            base.setPhone(userInfo.getMobile());
            base.setName(userInfo.getPatientName());
            if (Constant.STRING_YES.equals(userInfo.getCertificateType())) {
                base.setCertId(userInfo.getCertificate());
            }
            base.setVerifyStatus(Constant.STRING_NO);
            // 身份证号合法，设置年龄、性别、实名认证标志
            if (IdcardUtil.isValidCard(base.getCertId())) {
                String certId = base.getCertId();
                base.setAge(String.valueOf(IdcardUtil.getAgeByIdCard(certId)));
                base.setGender(Constant.INT_YES.equals(IdcardUtil.getGenderByIdCard(certId)) ? "男" : "女");
                base.setVerifyStatus(Constant.STRING_YES);
            }

            // 卡号卡类型
            base.setMediCardType(userInfo.getMediCardType());
            base.setMediCardId(userInfo.getMediCardType());
            this.save(base);
            return base;
        }
        return null;
    }

    /**
     * 保存中智公众号用户信息
     *
     * @param current  当前用户
     * @param userInfo 公众号用户详情
     * @return XSZYUserInfo {@link WOAZzUserInfo}
     */
    @Override
    public HphyPatientBase saveWoaZzInfo(CurrentUser current, WOAZzUserInfo userInfo) {
        if (ObjectUtil.isNotNull(userInfo)) {
            HphyPatientBase base = new HphyPatientBase();
            base.setUserId(current.getId());
            base.setOpenId(current.getOpenId());
            base.setPhone(userInfo.getMobile());
            base.setName(userInfo.getUsername());
            base.setCertId(userInfo.getPersoncard());
            base.setGender(Constant.BYTE_YES.equals(userInfo.getGender()) ? "男" : "女");
            base.setVerifyStatus(Constant.STRING_NO);
            // 身份证号合法，设置年龄、性别、实名认证标志
            if (IdcardUtil.isValidCard(base.getCertId())) {
                String certId = base.getCertId();
                base.setAge(String.valueOf(IdcardUtil.getAgeByIdCard(certId)));
                base.setVerifyStatus(Constant.STRING_YES);
            }
            this.save(base);
            return base;
        }
        return null;
    }

    /**
     * 保存万达公众号用户信息
     *
     * @param current  当前用户
     * @param userInfo 公众号用户详情
     * @return XSZYUserInfo {@link WOAWDUserInfo}
     */
    @Override
    public HphyPatientBase saveWoaWdInfo(CurrentUser current, WOAWDUserInfo userInfo) {
        if (ObjectUtil.isNotNull(userInfo)) {
            HphyPatientBase base = new HphyPatientBase();
            base.setUserId(current.getId());
            base.setOpenId(current.getOpenId());
            base.setPhone(userInfo.getMobile());
            base.setName(userInfo.getUsername());
            base.setCertId(userInfo.getPersoncard());
            base.setGender(Constant.BYTE_YES.equals(userInfo.getGender()) ? "男" : "女");
            base.setVerifyStatus(Constant.STRING_NO);
            // 身份证号合法，设置年龄、性别、实名认证标志
            if (IdcardUtil.isValidCard(base.getCertId())) {
                String certId = base.getCertId();
                base.setAge(String.valueOf(IdcardUtil.getAgeByIdCard(certId)));
                base.setVerifyStatus(Constant.STRING_YES);
            }
            this.save(base);
            return base;
        }
        return null;
    }

    @Override
    public void savePatient(HphyPatientBase patient) {
        HphyPatientBase base = this.getByCertId(patient.getCertId());
        if (ObjectUtil.isNull(base) || NumberUtil.isNullOrZero(base.getId())) {
            base = this.getByPhone(patient.getPhone(), false);
        }
        if (ObjectUtil.isNotNull(base) && NumberUtil.isNotNullOrZero(base.getId())) {
            patient.setId(base.getId());
            this.updateById(patient);
        } else {
            this.save(patient);
        }
    }

    private HphyPatientBase getByCertId(String certId) {
        LambdaQueryWrapper<HphyPatientBase> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(HphyPatientBase::getCertId, certId)
                .orderByDesc(HphyPatientBase::getUpdateTime)
                .last("limit 1");

        return this.getOne(wrapper);
    }

    @Override
    public HphyPatientBase getCurrentBase() {
        CurrentUser current = SecurityUtils.getCurrentUser();
        return this.getByOpenId(current.getOpenId());
    }
}
