package cn.sh.stc.sict.theme.hphy.util;

import cn.hutool.core.collection.CollUtil;
import cn.sh.stc.sict.theme.hpgp.model.HpgpBusyIdlePrediction;
import lombok.experimental.UtilityClass;
import lombok.extern.slf4j.Slf4j;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @author gao
 * @date 2023/03/21 17:27
 */
@Slf4j
@UtilityClass
public class BusyIdlePredictionUtil {

    /**
     *  获取有号源的三个科室
     * @param hpgpBusyIdlePredictionList
     * @return
     */
    public static List<HpgpBusyIdlePrediction> getHasRemainNum(List<HpgpBusyIdlePrediction> hpgpBusyIdlePredictionList) {
        if (CollUtil.isEmpty(hpgpBusyIdlePredictionList)) {
            return Collections.emptyList();
        }

        // 当前医院近七天有号的三个科室
        List<String> deptCodeList = hpgpBusyIdlePredictionList.stream()
                // 按科室分组，并统计每个科室可预约号源总量
                .collect(Collectors.groupingBy(HpgpBusyIdlePrediction::getDeptCode,
                        Collectors.summingInt(item -> Optional.ofNullable(item.getRemainNum()).orElse(0))))
                .entrySet().stream()
                // 按可预约号源进行排序，取排名前三的科室
                .sorted(Collections.reverseOrder(Map.Entry.comparingByValue()))
                .map(Map.Entry::getKey)
                .limit(3)
                .collect(Collectors.toList());

        return hpgpBusyIdlePredictionList.stream().filter(item -> deptCodeList.contains(item.getDeptCode())).collect(Collectors.toList());

    }
}
